#!/bin/bash

# This is install/upgrade script for ksrs terminal software

baseip=0
if [ -n "$1" ] ; then
  baseip=$1
fi

MainKSRSDir=/opt/ksrs-terminal
KSRSServerDir=server
KSRSServerBin=krs-terminal-server
KSRSAgentDir=agent
KSRSAgentBin=snmp-agent
KSRSGUIDir=gui
KSRSLibDir=lib
TermKSRSUnits="../Terminal/Modules/Linux/Fedora"


if ! [ -d "../NEW_BUILD/x86-64-linux-qt5.15.9-release/Terminal" ]; then
qtver="qt5.15.14"
else
qtver="qt5.15.9"
fi

TermKSRSUnitsNew="../NEW_BUILD/x86-64-linux-$qtver-release/Terminal"
TermKSRSUnitsLib="../NEW_BUILD/x86-64-linux-$qtver-release/Lib"


TermLIRADrv="../Terminal/Drv/Linux"
TermLIRADrvBlacklist="../Terminal/Drv/Linux/lemz.conf"
TermLIRAModules="$TermLIRADrv/modules"
TermKSRSLibs="/etc/ld.so.conf.d/ksrs-terminal.conf"
Dependencies="../Dependencies/Linux"
SysctlConfigFile="99-ksrs-network-sysctl.conf"
KSRSCollectorTrapFwdFile="process-trap-forward.php"
KSRSCollectorTrapFwd="../Dependencies/Common/$KSRSCollectorTrapFwdFile"
KSRSTermServiceName=lemz-ksrs-terminal
KSRSTermService="/usr/lib/systemd/system/$KSRSTermServiceName.service"
KSRSTermServiceContent="[Unit]
Description=System KSRS terminal Service
Requires=network.target
After=NetworkManager-wait-online.service
After=network-online.target

[Service]
Type=simple
Environment=LOGNAME=root
ExecStart=$MainKSRSDir/$KSRSServerDir/$KSRSServerBin --run
ExecStop=/usr/bin/killall -w -SIGINT $KSRSServerBin
Restart=on-failure
StandardOutput=null

[Install]
WantedBy=multi-user.target"
KSRSTermAgentName=lemz-snmp-agent
KSRSTermAgent="/usr/lib/systemd/system/$KSRSTermAgentName.service"
KSRSTermAgentContent="[Unit]
Description=System SNMP KSRS Agent Service
Requires=network.target
After=network.target runlevel1.target

[Service]
Type=simple
ExecStart=$MainKSRSDir/$KSRSAgentDir/$KSRSAgentBin --run
ExecStop=/usr/bin/killall -w -SIGINT $KSRSAgentBin
Restart=on-failure
StandardOutput=null

[Install]
WantedBy=multi-user.target"

SNMPRegex="^net-snmp-[0-9]+"

SNMPTrapServiceName="snmptrapd"
KSRSsnmptrapdConfContent="ignoreAuthFailure yes
disableAuthorization yes
snmpTrapdAddr UDP:162
traphandle default /usr/bin/php /usr/bin/$KSRSCollectorTrapFwdFile"
SNMPTRAPDMAINPID='$MAINPID'
KSRSSNMPTRAPDSevice="/usr/lib/systemd/system/$SNMPTrapServiceName.service"
SNMPTRAPDServiceContent="[Unit]
Description=Simple Network Management Protocol (SNMP) Trap Daemon.
After=syslog.target network.target

[Service]
Type=notify
EnvironmentFile=-/etc/sysconfig/snmptrapd
ExecStart=/usr/sbin/snmptrapd -Ln -f -One -n
ExecReload=/bin/kill -HUP $SNMPTRAPDMAINPID

[Install]
WantedBy=multi-user.target"


SNMPServiceName="snmpd"
KSRSTermOID=".1.3.6.1.4.1.120312.3"
KSRSsnmpdConModule="process-terminal-agent.php"
KSRSsnmpdConModuleSRC="../Dependencies/Common/$KSRSsnmpdConModule"
KSRSsnmpdConfContent="trap2sink 192.168.12.$((baseip+1)) public
trap2sink 192.168.12.$((baseip+2)) public
rwcommunity public 192.168.12.$((baseip+1))
rwcommunity public 192.168.12.$((baseip+2))
disk /
disk /opt
createUser internal
rouser internal
agentSecName internal
iquerySecName internal
defaultMonitors yes
notificationEvent  linkUpTrap    linkUp   ifIndex ifAdminStatus ifOperStatus
notificationEvent  linkDownTrap  linkDown ifIndex ifAdminStatus ifOperStatus
monitor  -r 10 -e linkUpTrap   \"Generate linkUp\"   ifOperStatus != 2
monitor  -r 10 -e linkDownTrap \"Generate linkDown\" ifOperStatus == 2
pass_persist $KSRSTermOID /usr/bin/php /usr/bin/$KSRSsnmpdConModule"
SNMPDMAINPID='$MAINPID'
KSRSSNMPAgent="/usr/lib/systemd/system/$SNMPServiceName.service"
SNMPDServiceContent="[Unit]
Description=Simple Network Management Protocol (SNMP) Daemon.
After=syslog.target network.target

[Service]
Type=notify
EnvironmentFile=-/etc/sysconfig/snmpd
ExecStart=/usr/sbin/snmpd -Ln -f
ExecReload=/bin/kill -HUP $SNMPDMAINPID

[Install]
WantedBy=multi-user.target"

KSRSGUIUser=user0
KSRSAutoGUI=KARSTerminal.desktop
KSRSAutoGUIContent="[Desktop Entry]
Type=Application
Exec=/opt/ksrs-terminal/gui/KARSTerminal 2>/dev/null
Hidden=false
NoDisplay=false
X-GNOME-Autostart-enabled=true
Name[ru_RU]=KSRS Terminal
Name=KSRS Terminal
Comment[ru_RU]=
Comment="

GUIServiceName="lemz-ksrs-terminal-gui"

GUIServiceNameOld="ksrs-terminal-gui"
KSRSGUIServiceOld="/usr/lib/systemd/system/$GUIServiceNameOld.service"

KSRSGUIService="/usr/lib/systemd/system/$GUIServiceName.service"
GUIServiceContent="[Unit]
Description=Start KSRS Terminal GUI.
After=display-manager.service network.service network-online.target

[Service]
User=user0
Group=user0
Type=simple
Environment=DISPLAY=:0
Environment=XAUTHORITY=/home/user0/.Xauthority
ExecStart=/opt/ksrs-terminal/gui/KARSTerminal --service
Restart=on-failure
StandardOutput=null

[Install]
WantedBy=graphical.target"

KSRSAutoLoginContent="[base]
autologin=user0
session=/usr/bin/openbox-session
greeter=/usr/libexec/lxdm-greeter-gtk

[server]
arg=/usr/bin/X -nocursor -background none vt1

[display]
gtk_theme=Clearlooks
bg=/usr/share/backgrounds/default.png
bottom_pane=1
lang=1
keyboard=0
theme=Industrial

[input]

[userlist]
disable=0
white=
black=
"

TermCronConf="/etc/cron.d/cron_ksrs_term"
TermKSRSScripts=scripts

KSRScronConfContent="
SHELL=/bin/bash
PATH=/sbin:/bin:/usr/sbin:/usr/bin
MAILTO=root
0 3 * * * root $MainKSRSDir/$TermKSRSScripts/remove_logs.sh"


# 0. Check for root access
printf "checking for user access.. "
if [ $(whoami) != "root" ]; then
  echo "fail [need root access]"
  exit 1
else
  echo "ok"
fi

hostip=$(ifconfig | grep -Eo 'inet (addr:)?([0-9]*\.){3}[0-9]*' | grep -Eo '([0-9]*\.){3}[0-9]*' | grep -v '127.0.0.1')
echo "host ip is $hostip"
for ((ip = 1 ; ip <= 5 ; ip++)); do
if [[ $hostip == $((baseip+ip)) ]]; then
echo "is tech host, terminal software cannot be installed"
sleep 5
exit 1
fi
done
#for TechIP in "192.168.12.1" "192.168.12.2" "192.168.12.3" "192.168.12.4" "192.168.12.5"
#do
#if [[ $hostip == $TechIP ]]; then
#echo "is tech host, terminal software cannot be installed"
#sleep 5
#exit 1
#fi
#done



# 1. Check dependencies
#look packages
printf "checking for rpm dependences.. "
rpmList=$(rpm -qa | grep -e qt-4.8 -e qt-x11 -e net-snmp -e php -e sddm -e openbox -e qt5-qtmultimedia -e qt5-qtwebengine -e kernel-lt-dev)
rpmQTMain=0
rpmQTX11=0
rpmPHP=0
rpmSNMP=0
rpmSNMPAgent=0
rpmSNMPUtils=0
rpmLXDM=0
rpmOPENBOX=0
rpmQTMultimedia=0
rpmQTWebengine=0
rpmKernelDev=0
for aPacket in $rpmList; do
  if [ "$rpmQTMain" -eq 0 ]; then
    if [[ $aPacket == qt-4.8*.x86_64 ]]; then
      rpmQTMain=1
      continue
    fi
  fi
  if [ "$rpmQTX11" -eq 0 ]; then
    if [[ $aPacket == qt-x11*.x86_64 ]]; then
      rpmQTX11=1
      continue
    fi
  fi
  if [ "$rpmPHP" -eq 0 ]; then
    if [[ $aPacket == php*.x86_64 ]]; then
      rpmPHP=1
      continue
    fi
  fi
  if [ "$rpmSNMP" -eq 0 ]; then
    if [[ $aPacket =~ $SNMPRegex ]]; then
      rpmSNMP=1
      continue
    fi
  fi
  if [ "$rpmSNMPAgent" -eq 0 ]; then
    if [[ $aPacket == net-snmp-agent-libs*.x86_64 ]]; then
      rpmSNMPAgent=1
      continue
    fi
  fi  
  if [ "$rpmSNMPUtils" -eq 0 ]; then
    if [[ $aPacket == net-snmp-utils*.x86_64 ]]; then
      rpmSNMPUtils=1
      continue
    fi
  fi
  if [ "$rpmLXDM" -eq 0 ]; then
    if [[ $aPacket == sddm.x86_64 ]]; then
      rpmLXDM=1
      continue
    fi
  fi
  if [ "$rpmOPENBOX" -eq 0 ]; then
    if [[ $aPacket == openbox-* ]]; then
      rpmOPENBOX=1
      continue
    fi
  fi
   if [ "$rpmQTMultimedia" -eq 0 ]; then
    if [[ $aPacket == qt5-qtmultimedia*.x86_64 ]]; then
      rpmQTMultimedia=1
      continue
    fi
  fi
   if [ "$rpmQTWebengine" -eq 0 ]; then
    if [[ $aPacket == qt5-qtwebengine*.x86_64 ]]; then
      rpmQTWebengine=1
      continue
    fi
  fi
   if [ "$rpmKernelDev" -eq 0 ]; then
    if [[ $aPacket == kernel-lt-devel.x86_64 ]]; then
      rpmKernelDev=1
      continue
    fi
  fi

done
if [ "$rpmQTMain" -eq 0 -o "$rpmQTX11" -eq 0 -o "$rpmPHP" -eq 0 -o "$rpmSNMP" -eq 0 -o "$rpmSNMPAgent" -eq 0 -o "$rpmSNMPUtils" -eq 0 -o "$rpmOPENBOX" -eq 0 -o "$rpmQTMultimedia" -eq 0 -o "$rpmQTWebengine" -eq 0 ]; then
  echo "fail"
  if [ "$rpmQTMain" -eq 0 ]; then
    echo "package <qt> is not accessible"
  fi
  if [ "$rpmQTX11" -eq 0 ]; then
    echo "package <qt-x11> is not accessible"
  fi
  if [ "$rpmPHP" -eq 0 ]; then
    echo "package <php> is not accessible"
  fi
  if [ "$rpmSNMP" -eq 0 ]; then
    echo "package <net-snmp> is not accessible"
  fi
  if [ "$rpmSNMPAgent" -eq 0 ]; then
    echo "package <net-snmp-agent> is not accessible"
  fi  
  if [ "$rpmSNMPUtils" -eq 0 ]; then
    echo "package <net-snmp-utils> is not accessible"
  fi
#  if [ "$rpmLXDM" -eq 0 ]; then
#    echo "package <sddm> is not accessible"
#  fi
  if [ "$rpmOPENBOX" -eq 0 ]; then
    echo "package <openbox> is not accessible"
  fi
  if [ "$rpmQTMultimedia" -eq 0 ]; then
#    sudo  dnf -y install qt5-qtmultimedia.x86_64 2>/dev/null
    echo "package <qt5-qtmultimedia> is not accessible"
  fi
  if [ "$rpmQTWebengine" -eq 0 ]; then
    echo "package <qt5-qtwebengine> is not accessible"
  fi
#  if [ "$rpmKernelDev" -eq 0 ]; then
#    echo "package <kernel-lt-dev> is not accessible"
#  fi
  exit 1
fi
echo "ok"
#look kernel module

## system scripts
trim() 
{
 local var="$*"
 var="${var#"${var%%[![:space:]]*}"}"
 var="${var%"${var##*[![:space:]]}"}"
 printf '%s' "$var"
}
printf "copy sysctl configs.. "
if [ $(cp -f "$Dependencies/$SysctlConfigFile" /etc/sysctl.d) ]; then
  echo "fail"
  #exit 1
else
  echo "ok"
  chmod -f 644 /etc/sysctl.d/$SysctlConfigFile
  #apply immediately
  while IFS='=' read -r pname pval; do
    pname=$(trim $pname)
    [[ $pname = \#* ]] && continue
    pval=$(trim $pval)
    sysctl -w "$pname=$pval"
  done < /etc/sysctl.d/$SysctlConfigFile
fi

# 2. (re)Place binary files
#create main binary directory (if need)
if ! [ -d $MainKSRSDir ]; then
  printf "create main directory for binary ($MainKSRSDir).. "
  if [ $(mkdir -m755 $MainKSRSDir) ]; then
    echo "fail"
    exit 1
  else
    echo "ok"
  fi
fi

#create server directory (if need)
if ! [ -d "$MainKSRSDir/$KSRSServerDir" ]; then
  printf "create server directory for binary ($MainKSRSDir/$KSRSServerDir).. "
  if [ $(mkdir -m755 $MainKSRSDir/$KSRSServerDir) ]; then
    echo "fail"
    exit 1
  else
    echo "ok"
  fi
fi

#copy files
#server
ksrsFiles=$(ls $TermKSRSUnits/$KSRSServerDir)
for afile in $ksrsFiles; do
  printf "copy server file ($afile).. "
  if [ $(cp -fR $TermKSRSUnits/$KSRSServerDir/$afile $MainKSRSDir/$KSRSServerDir) ]; then
    echo "fail"
    exit 1
  else
    echo "ok"
    printf "chmod server file ($afile) to 755.. "
    chmod -f 755 $MainKSRSDir/$KSRSServerDir/$afile
    if [ $? -ne 0 ]; then
      echo "fail"
      exit 1
    else
      echo "ok"
    fi
  fi
done

#server New
#ksrsFiles=$(ls $TermKSRSUnitsNew/$KSRSServerDir)
#for afile in $ksrsFiles; do
  printf "copy server file (krs-terminal-server).. "
  if [ $(cp -fR $TermKSRSUnitsNew/krs-terminal-server $MainKSRSDir/$KSRSServerDir) ]; then
    echo "fail"
    exit 1
  else
    echo "ok"
    printf "chmod server file (krs-terminal-server) to 755.. "
    chmod -f 755 $MainKSRSDir/$KSRSServerDir/krs-terminal-server
    if [ $? -ne 0 ]; then
      echo "fail"
      exit 1
    else
      echo "ok"
    fi
  fi
#done







# 5. (re)Install services (if needed)
#stop if running and autorun
runStatus=$(systemctl is-active $KSRSTermServiceName)
if [ "$runStatus" == "active" ]; then
  printf "stop $KSRSTermServiceName service.. "
  if [ $(systemctl stop $KSRSTermServiceName) ]; then
    echo "fail"
    exit 1
  fi
  echo "ok"
fi
autoStatus=$(2>/dev/null systemctl is-enabled $KSRSTermServiceName)
if [ "$autoStatus" == "enabled" ]; then
  printf "disable $KSRSTermServiceName service.. "
  if [ $(systemctl -q disable $KSRSTermServiceName) ]; then
    echo "fail"
    exit 1
  fi
  echo "ok"
fi


#make server service configuration
if ! [ -f $KSRSTermService ]; then
  printf "create ($KSRSTermService).. "
else
  printf "renew ($KSRSTermService).. "
fi
if [ $(echo "$KSRSTermServiceContent" > $KSRSTermService) ]; then
  echo "fail"
  exit 1
fi
echo "ok"

#apply changes
printf "reload systemd daemons configuration.. "
if [ $(systemctl daemon-reload) ]; then
  echo "fail"
  exit 1
fi
echo "ok"
#setup autorun
printf "enable $KSRSTermServiceName service.. "
if [ $(systemctl -q enable $KSRSTermServiceName) ]; then
  echo "fail"
  exit 1
fi
echo "ok"

#start services
printf "starting $KSRSTermServiceName service.. "
if [ $(systemctl start $KSRSTermServiceName) ]; then
  echo "fail"
  exit 1
fi
echo "ok"

# 6. autologin and autostart





echo "Terminal software setup completed successfully. Please reboot operating system!"
# normal exit
exit 0
